/* The LinTalk Project
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/times.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h>

#ifndef HAVE_BANLIST_H
#define HAVE_BANLIST_H

#define BL_PERIOD_LEN 120     /* The period in which the number of connects are counted in seconds. */
#define BL_MAXCONNECTS 120    /* The number of maximum connects in period. */
#define BL_BLOCKTIME 60       /* The time to block a host that connected too often. */

typedef struct {
  struct in_addr;             /* The address of the host. */
  GList *connecttimes;        /* A list of times when the hosted connected us. */
  time_t lastconnect;         /* The time when the host last connected. */
  int numconnects;            /* Since we don't want to count the list length all */
                              /* the time, we store the number of connects in here. */
} BanlistHost;

typedef struct {
  int periodlen;      /* The period in which the number of connects are counted in seconds. */
  int maxconnects;    /* The number of maximum connects in period. */
  int blocktime;      /* The time to block a host that connected too often. */
  GList * hosts;      /* A list of hosts that tried to connect us in the last period. */
} Banlist;

Banlist * banlist_create(void);
void banlist_destroy(Banlist *bl);
short int banlist_check(Banlist *bl, struct sockaddr_in * client_address);

#endif
