/* The LinTalk Project
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <pthread.h>
#include <stdio.h>
#include <string.h>
#include "callbacks.h"
#include "gui_other.h"
#include "gui_icons.h"
#include "support.h"
#include "configfile.h"
#include "gui_users.h"
#include "gui_userlist.h"
#include "gui_preferences.h"
#include "gui_talk.h"
#include "globals.h"

extern pthread_mutex_t mut;
extern Cfg cfg;
extern GUI gui;


gboolean
on_treeview_users_button_press_event   (GtkWidget       *tree,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  GtkTreeModel *model = NULL;
  GtkTreePath *userpath = NULL;
  GtkTreeIter iter;
  User *user = NULL;
  int type = 0;
  
  switch (event->type) {
  case GDK_2BUTTON_PRESS:
    gtk_tree_view_get_path_at_pos((GtkTreeView *)tree, event->x, event->y, &userpath, NULL, NULL, NULL);
    model = (GtkTreeModel *)gtk_tree_view_get_model((GtkTreeView *)tree);
  
    if (!gtk_tree_model_get_iter(model, &iter, userpath))
      return(FALSE);
    
    gtk_tree_path_free(userpath);
    
    gtk_tree_model_get(model, &iter, 1, &user, 2, &type, -1);
    if (type == 0)
      return(FALSE);
    
    if (!user)
      return(FALSE);
    
    talk_to(user);
    
    break;
    
  default:
    break;
  }
  
  return(FALSE);
}


void
on_button_menu_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_menu_popup(GTK_MENU(gui.menu), NULL, NULL, NULL, NULL, 1, 0);
}


void
on_button_status_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  /* Toggle online status. */
  if (cfg.local.status == OFFLINE) {
    cfg.local.status = AVAILABLE;
    icons_status_set(AVAILABLE);
  } else {
    cfg.local.status = OFFLINE;
    icons_status_set(OFFLINE);
  }
}


void
on_menu_add_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show(gui.newuser);
}


void
on_menu_remove_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  userlist_delete_selected();
}


void
on_menu_preferences_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  preferences_switch(NULL);
}


void
on_lintalk_destroy                     (GtkObject       *object,
                                        gpointer         user_data)
{
  extern pthread_mutex_t mut;

  /* Send shutdown signal. */
  pthread_mutex_lock(&mut);
  cfg.shutdown = 1;
  pthread_mutex_unlock(&mut);
}


void
on_button_newuser_close_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide(gui.newuser);
}


void
on_button_newuser_add_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
  newuser_add_selected();
  gtk_widget_hide(gui.newuser);
}


void
on_button_pref_close_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide(gui.preferences);
}


int on_sleep_timeout (void *null)
{
  if (!cfg.shutdown) {
    userlist_update();
    newuser_lists_update();
  }
  gtk_timeout_add(GUI_UPDATE_INTERVAL, on_sleep_timeout, NULL);
  return (0);
}


void
on_entry_newuser_group_activate        (GtkEntry        *entry,
                                        gpointer         user_data)
{
  newuser_add_group(gui.newuser);
}


void
on_button_newuser_group_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{
  newuser_add_group(gui.newuser);
}


void
on_entry_newuser_host_activate         (GtkEntry        *entry,
                                        gpointer         user_data)
{
  newuser_add(gui.newuser);
}


void
on_button_newuser_newuser_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
  newuser_add(gui.newuser);
}


gboolean
on_tree_preferences_group_button_press_event
                                        (GtkWidget       *tree,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  GtkTreePath *userpath = NULL;
  
  switch (event->button) {
  case 1:
    gtk_tree_view_get_path_at_pos((GtkTreeView *)tree, event->x, event->y, &userpath, NULL, NULL, NULL);
    
    if (!userpath)
      return(FALSE);
    
    preferences_switch(userpath);
    return(FALSE);
    break;
    
  default:
    break;
  }
  
  return(TRUE);
}

gboolean
on_textview_talk_text_key_pressed  (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer        data)
{
  User *user = data;
  
  /* Catch the return key. (which means: send) */
  if (event->keyval == 65293) {
    talk_send(user);
    
    gtk_widget_destroy(user->talk);
    user->talk = NULL;
    
    return(TRUE);
  }
  
  return(FALSE);
}


void
on_button_talk_cancel_clicked          (GtkButton       *button,
                                        gpointer        data)
{
  User *user = data;

  gtk_widget_destroy(user->talk);
  user->talk = NULL;
}


void
on_button_talk_send_clicked            (GtkButton       *button,
                                        gpointer        data)
{
  User *user = data;
  
  talk_send(user);
  gtk_widget_destroy(user->talk);
  user->talk = NULL;
}


void on_talk_destroy (GtkWidget * widget, gpointer data)
{
  User *user = data;
  
  gtk_widget_destroy(user->talk);
  user->talk = NULL;
}
