/* The LinTalk Project
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */  
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "globals.h"
#include "network.h"
#include "client.h"
#include "users.h"
extern pthread_mutex_t mut;
extern Cfg cfg;

/***************************************************************************************
 * BELOW FOLLOW THE STATICS
 ***************************************************************************************/
/*
 * Send all messages in the message queue to a user.
 */
static void send_queue(User * user)
{
  char buffer_out[2048] = "";
  Historyentry *histentry = NULL;
  char *text = NULL;
  int sd = 0;
  GList *item = NULL;

  /* Connect to the remote user's client. */
  if ((sd = connect_host(user->host, LINTALK_PORT)) <= 0)
    goto error;

  /* Mutex locked. */
  pthread_mutex_lock(&mut);

  /* Parse through the out-queue of the user. */
  item = user->outqueue;
  while (item != NULL) {
    text = item->data;

    /* Prepare outgoing message. */
    snprintf(buffer_out, 2047, "MSG %s\n%s\n", cfg.local.host, text);
#ifdef _DEBUG_
    printf(">> %s\n", buffer_out);
#endif

    /* Unlock mutex while sending via network. */
    pthread_mutex_unlock(&mut);

    /* send MSG. */
    if (write(sd, buffer_out, strlen(buffer_out)) <= 0) {
      fprintf(stderr, "Error while sending Message!\n");
      goto error;
    }
    /* Re-lock mutex. */
    pthread_mutex_lock(&mut);

    /* Remember the sent message in the user history. */
    histentry = malloc (sizeof(Historyentry));
    histentry->user = 0;
    histentry->username = strdup (cfg.local.name);
    histentry->text = strdup (text);
    
    user->history = g_list_append(user->history, histentry);

    /* Next item. */
    item = item->next;
  }
  /* The queue must now be empty. (yes, the mutex is still locked) */
  g_list_free(user->outqueue);
  user->outqueue = NULL;

  /* Now, unlock. */
  pthread_mutex_unlock(&mut);

  /* Close network socket and return without an error. */
  close(sd);
  return;

  /* In case of a network error, we end up here. */
error:
  /* The mutex was unlocked while networking, so we lock it to change the user properties. */
  pthread_mutex_lock(&mut);

  /* User no longer available. */
  user->status = OFFLINE;

  /* We're done. Unlock. */
  pthread_mutex_unlock(&mut);

  /* Close socket and return. */
  close(sd);
  return;
}

/***************************************************************************************
 * END OF STATICS
 ***************************************************************************************/

/*
 * In this loop all open send-actions are taken.
 * It is started once at program startup and stays alive all the time.
 */
void *sendloop(void *psd)
{
  GList *item = NULL;
  User *user = NULL;

  /* We have all known hosts in our userlist, so... */
  while (!cfg.shutdown) {
    /* If the users chose not to be online, don't send anything. */
    if (cfg.local.status == OFFLINE) {
      usleep(500);
      continue;
    }
    
    /* ...send all queued messages to them. */
    item = cfg.users;
    while (item != NULL) {
      pthread_mutex_lock(&mut);
      user = item->data;

      /* Check if there is something to send in the out queue. */
      if (user->outqueue != NULL) {
        pthread_mutex_unlock(&mut);
        send_queue(user);
      } else {
        pthread_mutex_unlock(&mut);
      }

      item = item->next;
    }

    /* We check for queue items every half second. */
    usleep(500);
  }
  
  /* Close posix thread. */
  pthread_exit(0);
}
