/* The LinTalk Project
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include "globals.h"
#include "groups.h"

extern pthread_mutex_t mut;
extern Cfg cfg;


/***************************************************************************************
 * STATICS
 ***************************************************************************************/
/***************************************************************************************
 * END STATICS
 ***************************************************************************************/


Group * group_add(const char * name)
{
  Group *group = malloc(sizeof(Group));
  group->name = name;
  pthread_mutex_lock(&mut);
  cfg.groups = g_list_append(cfg.groups, group);
  group->number = g_list_index(cfg.groups, group);
  pthread_mutex_unlock(&mut);
  return(group);
}


void group_remove(int number)
{
  Group *group = NULL;
  
  pthread_mutex_lock(&mut);
  if (!(group = group_find_from_number(number)))
    return;
  cfg.groups = g_list_remove(cfg.groups, group);
  free(group);
  pthread_mutex_unlock(&mut);
}


Group * group_find_from_number(int number)
{
  Group *curgroup = NULL;
  GList * item = NULL;
  
  /* Find the group. */
  item = cfg.groups;
  while (item) {
    curgroup = (Group *)item->data;
    if (curgroup->number == number)
      return(curgroup);
    item = item->next;
  }
  
  return(NULL);
}


Group * group_find_from_name(const char * name)
{
  Group *curgroup = NULL;
  GList *item = NULL;
  
  /* Find the group. */
  item = cfg.groups;
  while (item) {
    curgroup = (Group *)item->data;
    if (strcmp(curgroup->name, name) == 0)
      return(curgroup);
    item = item->next;
  }
  
  return(NULL);
}
