/* The LinTalk Project
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gtk/gtk.h>
#include "globals.h"
#include "support.h"
#include "gui_icons.h"

extern GUI gui;

char * statusnames[] = {
  "Offline",
  "Available",
  "Invalid"
};


/*
 * Change the icon of the "Status" button in the mainwindow.
 */
void icons_status_set(gint status)
{
  GtkImage * image = (GtkImage *)lookup_widget(gui.lintalk, "image_status");
  GtkVBox *vbox = (GtkVBox *)lookup_widget(gui.lintalk, "vbox_status");
  GtkLabel *label = (GtkLabel *)lookup_widget(gui.lintalk, "label_status");
  
  /* Destroy the old icon and remove the label. */
  gtk_container_remove((GtkContainer *)vbox, (GtkWidget *)image);
  gtk_container_remove((GtkContainer *)vbox, (GtkWidget *)label);
  gtk_widget_destroy((GtkWidget *)image);
  
  /* Create a new icon. */
  switch (status) {
  case OFFLINE:
    image = (GtkImage *)gtk_image_new_from_stock("gtk-dialog-error", GTK_ICON_SIZE_LARGE_TOOLBAR);
    break;
  
  case AVAILABLE:
    image = (GtkImage *)gtk_image_new_from_stock("gtk-apply", GTK_ICON_SIZE_LARGE_TOOLBAR);
    break;
  
  default:
    image = (GtkImage *)gtk_image_new_from_stock("gtk-dialog-question", GTK_ICON_SIZE_LARGE_TOOLBAR);
    break;
  }
  
  /* Pack new icon. */
  gtk_widget_set_name((GtkWidget *)image, "image_status");
  gtk_widget_show((GtkWidget *)image);
  gtk_box_pack_start((GtkBox *)vbox, (GtkWidget *)image, TRUE, TRUE, 0);
  g_object_set_data_full (G_OBJECT(gui.lintalk), "image_status", gtk_widget_ref((GtkWidget *)image), (GDestroyNotify) gtk_widget_unref);
  
  /* Pack new label. */
  gtk_label_set_text(label, statusnames[status]);
  gtk_box_pack_start((GtkBox *)vbox, (GtkWidget *)label, TRUE, TRUE, 0);
}


/*
 * Preload all icons.
 */
GList * icons_load(char * theme)
{
  return(NULL);
}
