/* The LinTalk Project
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include "globals.h"
#include "gui_init.h"
#include "init.h"
#include "configfile.h"
#include "cli.h"

Cfg cfg;


/***************************************************************************************
 * STATICS
 ***************************************************************************************/
/*
 * Malloc / Init memory.
 */
static short int init_environment(void)
{
  /* Configfile version. */
  cfg.version = 2;
  
  /* Init the configuration struct. */
  cfg.serversd = 0;
  cfg.shutdown = 0;
  cfg.users = NULL;
  cfg.groups = NULL;
  group_add("Userlist root");
  
  /* Init my own profile. */
  cfg.local.status = AVAILABLE;   /* By default, we choose to be online. */
  if (!(cfg.local.name = calloc(2048, sizeof(char)))
    || !(cfg.local.phone = calloc(2048, sizeof(char)))
    || !(cfg.local.host = calloc(2048, sizeof(char)))) {
    fprintf(stderr, "Error: init_environment(): Memory exhausted! (malloc local)\n");
    return(FALSE);
  }
  
  return(TRUE);
}

#define DEFAULT_DIR_MODE (S_IRWXU|S_IRWXG|S_IRWXO)
/*
 * Check if the given directory exists.
 * If it doesn't, create it.
 */
static short int checkdir(char * directory)
{
  struct stat buf;
    
  /* Check if the config directory already exists. */
  if ( stat(directory, &buf) != -1 )
    return(TRUE);
  
  fprintf(stderr, "Warning: checkdir(): Can't stat configuration directory (%s)!\n", directory);
  
  /* Create a new dir. */
  fprintf(stderr, "Info: checkdir(): Creating configuration directory (%s)!\n", directory);
  mkdir(directory, DEFAULT_DIR_MODE);
  
  /* We should be able to stat this now. */
  if ( stat(directory, &buf) == -1 ) {
    fprintf(stderr, "Error: checkdir(): Still can't stat the configuration directory! (%s)!\n", directory);
    return(FALSE);
  }
  
  return(TRUE);
}


/*
 * Load configfile options.
 */
static short int init_configfile(void)
{
  char dirname[2048];
  char filename[2048];
  
  snprintf(dirname, 2047, "%s/.lintalk/", getenv("HOME"));
  snprintf(filename, 2047, "%s/.lintalk/config", getenv("HOME"));
  
  /* Check if the config directory exists. */
  if (!checkdir(dirname)) {
    fprintf(stderr, "Error: init_configfile(): Still can't stat the configuration directory! (%s)!\n", dirname);
    return(FALSE);
  }
  
  /* Now, try to load the configfile options into the memory. */
  if (configfile_load(filename, &cfg) < 0)
    fprintf(stderr, "Warning: init_configfile(): Failed to load the configfile! (%s)!\n", filename);
  
  /* Load buddies. */
  if (configfile_users_load() < 0)
    fprintf(stderr, "Warning: init_configfile(): Failed to load the buddylist!\n");
  
  return(TRUE);
}
#undef DEFAULT_DIR_MODE


/*
 * Load configfile options.
 */
static short int init_configfile_stor(void)
{
  char dirname[2048];
  char filename[2048];
  
  snprintf(dirname, 2047, "%s/.lintalk/", getenv("HOME"));
  snprintf(filename, 2047, "%s/.lintalk/config", getenv("HOME"));
  
  /* Check if the config directory exists. */
  if (!checkdir(dirname)) {
    fprintf(stderr, "Error: init_configfile_stor(): Still can't stat the configuration directory! (%s)!\n", dirname);
    return(FALSE);
  }
  
  /* Save the configuration and the user list. */
  if (configfile_save(filename, &cfg) < 0)
    fprintf(stderr, "Warning: init_configfile_stor(): Failed to save configuration. (%s)!\n", filename);
  
  //FIXMEFIXME: Returncode! Filename?
  if (configfile_users_save() < 0)
    fprintf(stderr, "Warning: init_configfile_stor(): Failed to save the buddylist! (%s)!\n", dirname);
  
  return(TRUE);
}


/*
 * Check the current configuration for inconstencies / invalid values.
 */
static short int init_sanitycheck(Cfg * cfg)
{
  if (*(cfg->local.host) == '\0')
    strncpy(cfg->local.host, "127.0.0.1", 2047);
  
  if (cfg->version != 2)
    return(FALSE);
  
  return(TRUE);
}
/***************************************************************************************
 * END STATICS
 ***************************************************************************************/


/*
 * Init all prerequirements for the program.
 */
short int init_all(int argc, char *argv[])
{
  /* Init memory. */
  if (!init_environment())
    return(FALSE);
  
  /* Load configfile options. */
  if (!init_configfile())
    return(FALSE);
  
  /* Apply commandline options. */
  if (!cli_parse(&cfg, argc, argv))
    return(FALSE);
  
  /* Check the current configuration for validity. */
  if (!init_sanitycheck(&cfg))
    return(FALSE);
  
  /* Create widgets and such. */
  if (!init_gui())
    return(FALSE);
  
  return(TRUE);
}


/*
 * After shutdown. Save config, free all memory,...
 */
void init_destroy_all(void)
{
  init_configfile_stor();
  
  free(cfg.local.name);
  free(cfg.local.phone);
  free(cfg.local.host);
  
  users_free_all(cfg.users);
}
