/* lib_configfile
 * (c)2002, 2003 by Samuel Abels (sam@manicsadness.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */  
#ifndef DLL_H
#define DLL_H
typedef struct DLL_s {
  void *prev;
  void *data;
  void *next;
} DLL;
#endif

#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE 1
#endif

#ifndef LIB_CFG_H
#define LIB_CFG_H
enum CFGTYPES {
  INTEGER,
  INTEGERU16,
  STRING
};
#endif

/* Struct to hold the mapping between option name and other attributes. */
typedef struct {
  char *category;
  char *option;
  short int type;
  void *dest;
} OVPair;

/*
 * This function will grab the value of one option out of the config variable.
 */
short int configfile_option_get(const char *config, const char *category, const char *option, char *value);

/*
 * This function will grab the *integer* value of one option out of the config variable.
 */
short int configfile_option_get_integer(const char *config, const char *category, const char *option, int *value);

/*
 * This function will add one option to the config variable.
 */
short int configfile_option_add(char *config, const char *category, const char *option, const char *value);

/*
 * This function will add one *integer* option to the config variable.
 */
short int configfile_option_add_integer(char *config, const char *category, const char *option, int *value);

/*
 * Add a list of options to the config variable.
 */
void configfile_list_add(char *config, const char *category, const char *array[]);

/*
 * Get a list of options from the config variable.
 */
void configfile_list_get(char *config, const char *category, char *array[]);

/*
 * Add a list of options to the config variable.
 */
short int configfile_set_pairs(char *config, const OVPair ** optvals);

/*
 * Get a list of options from the config variable.
 */
short int configfile_get_pairs(const char *config, const OVPair ** optvals);

/*
 * This function will remove one complete category (and its contents) from the config variable.
 */
short int configfile_category_remove(char *config, const char *category);

/*
 * This function will read the complete configfile into a config variable
 */
int configfile_read(const char *fullfilename, char *config, int maxsize);

/*
 * This function will write the complete config variable to the config file.
 */
int configfile_write(const char *fullfilename, char *config);
