/* The LinTalk Project
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */  
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <pthread.h>
#include "support.h"
#include "globals.h"
#include "polling.h"
#include "server.h"
#include "client.h"
#include "users.h"
#include "configfile.h"
#include "shutdown.h"
#include "init.h"

pthread_mutex_t mut;
extern GUI gui;

/***************************************************************************************
 * STATICS
 ***************************************************************************************/
/*
 * This starts all pthreads (and the gui thread).
 */
static void incoming_loop(void)
{
  pthread_t tid_srv, tid_poll, tid_send;    /* variables to hold thread IDs. */

  /* This thread will keep on looking for incoming connections. */
  pthread_create(&tid_srv, NULL, serverloop, (void *) NULL);
  /* Polling thread (updates the userlist and the GUI). */
  pthread_create(&tid_poll, NULL, pollingloop, (void *) NULL);
  /* Client thread (send messages to other users). */
  pthread_create(&tid_send, NULL, sendloop, (void *) NULL);

  /* Check if server has been shut down. */
  gdk_threads_enter();
  gtk_timeout_add(800, check_if_server_shut, NULL);

  /* The GTK GUI. */
  gtk_main();
  gdk_threads_leave();
  
  /* Wait until all threads are cancelled. */
  pthread_join(tid_poll, NULL);
  pthread_join(tid_srv, NULL);
  pthread_join(tid_send, NULL);
}
/***************************************************************************************
 * END STATICS
 ***************************************************************************************/


/*
 * root of all
 */
int main(int argc, char *argv[])
{
#ifdef ENABLE_NLS
  bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain(PACKAGE);
#endif

  /* Init stuff. */
  gtk_set_locale();
  g_thread_init(NULL);
  gdk_threads_init();
  gtk_init(&argc, &argv);
  
  add_pixmap_directory(PACKAGE_DATA_DIR "/pixmaps");
  add_pixmap_directory(PACKAGE_SOURCE_DIR "/pixmaps");

  /* Init memory, widgets, load configuration options, ... */
  if (!init_all(argc, argv))
    exit(1);

  /* Show mainwindow. */
  gtk_widget_show(gui.lintalk);
  
  /* Init all threads and stuff. */
  incoming_loop();
  
  /* Cleanups. */
  init_destroy_all();
  
  return (0);
}
