/* The LinTalk Project
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */  
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <glib.h>
#include "globals.h"
#include "users.h"
#include "network.h"
extern pthread_mutex_t mut;
extern Cfg cfg;


/***************************************************************************************
 * STATICS
 ***************************************************************************************/
/***************************************************************************************
 * END STATICS
 ***************************************************************************************/


/*
 * Connect to a host.
 */
int connect_host(char *host, unsigned int host_port) 
{
  struct sockaddr_in socket_address;
  struct hostent *hostent;
  int socket_number;

  /* Hostname = myself? Then return. */ 
  if (strcmp(host, cfg.local.host) == 0)
    return(-1);
  
  /* Get a Socket number. */ 
  if ((socket_number = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
    perror("Error: connect_host(): Can't open INET socket!");
    return(-2);
  }

  /* Server address lookup. */
  if (!(hostent = gethostbyname(host))) {
    close(socket_number);
    return(-3);
  }
  
  socket_address.sin_family = AF_INET;
  
  memcpy(&socket_address.sin_addr, hostent->h_addr, hostent->h_length);
  socket_address.sin_port = htons(host_port);
  
  /* Connect to the server... */ 
  /* FIXMEFIXME: Banlist MUST WORK here also!!... */
  if (connect(socket_number, (struct sockaddr *) &socket_address, sizeof(socket_address)) == -1) {
    close(socket_number);
    return(-4);
  }

  return(socket_number);
}


/*
 * receive one line from a host
 */
int get_line(int socket, char *line, int maxlen)
{
  int i = 0;
  int foo = 0;
  
  /* We need to get char by char, to the first occurence of a line feed, so we get only one line. */ 
  memset(line, 0, maxlen);
  
  for (i = 0; i < maxlen; i++) {
    if (strlen(line) > 2 && *(line + i - 1) == '\n')
      break;
    foo = read(socket, line + i, 1);
  }


  /* Remove the linefeed. */ 
  while ((*(line + strlen(line) - 1) == '\n' || *(line + strlen(line) - 1) == '\r')  &&strlen(line) > 0)
    *(line + strlen(line) - 1) = 0;
  
  return (foo);
}


/*
 * Send a request to a host.
 */
short int send_request(int sd, char *request)
{
  /* Send request. */
  if (write(sd, request, strlen(request)) <= 0) {
    fprintf(stderr, "Error while sending request: %s\n", request);
    return (FALSE);
  }
#ifdef _DEBUG_
  fprintf(stderr, "Sending request (send_request): %s\n", request);
#endif
  
  return (TRUE);
}


/*
 * Receive a reply from a remote host.
 * The reply should have the format: "VARIABLE=VALUE".
 * If the variable name can't be found in the current line,
 * print out an error and return FALSE.
 */
short int get_answer(int sd, char *expected, char *destination, int maxlen)
{
  char buffer_in[maxlen + strlen(expected) + 1];

  buffer_in[0] = '\0';

  /* Read one line. */ 
  if (get_line(sd, buffer_in, sizeof(buffer_in)) <= 0) {
    fprintf(stderr, "Error while receiving request.\n");
    return(FALSE);
  }
#ifdef _DEBUG_
  fprintf(stderr, "<< %s\n", buffer_in);
#endif
  
  /* Check reply. */
  if (strncmp(buffer_in, expected, strlen(expected)) != 0) {
#ifdef _DEBUG_
    fprintf(stderr, "Expected: %s\n", expected);
    fprintf(stderr, "but received: %s\n", buffer_in);
#endif
    return (FALSE);
  }

  /* Copy the value to the destination variable. */
  strncpy(destination, &buffer_in[strlen(expected)], maxlen);

  return (TRUE);
}


/*
 * Receive the reply of an "WHO" request.
 */
void receive_who(User * user, char *host)
{
  char buffer_name[2048] = "";
  char buffer_phone[2048] = "";
  char buffer_host[2048] = "";
  char buffer_status[2048] = "";
  char buffer_out[200] = "";
  int sd = 0;

  /* Connect. */
  if ((sd = connect_host(host, LINTALK_PORT)) <= 0)
    goto error;

  /* Prepare WHO request. */
  snprintf(buffer_out, 199, "WHO %s\n", cfg.local.host);

  /* Send request. */
  if (!send_request(sd, buffer_out))
    goto error;

  /* Read one line. (name) */ 
  if (!get_answer(sd, "NAME=", buffer_name, 2047))
    goto error;
  /* Next line (phone number). */
  if (!get_answer(sd, "PHONE=", buffer_phone, 2047))
    goto error;
  /* Next line (host name). */
  if (!get_answer(sd, "HOST=", buffer_host, 2047))
    goto error;
  /* Next line (User status). */
  if (!get_answer(sd, "STATUS=", buffer_status, 2047))
    goto error;
  
  /* Replace all user variables. (must lock mutex) */
  pthread_mutex_lock(&mut);

  /* Kill old values. */
  free(user->name);
  free(user->phone);
  free(user->host);
  /* New values. */
  user->name = strdup(buffer_name);
  user->phone = strdup(buffer_phone);
  user->host = strdup(buffer_host);
  user->status = user_status_char_to_int(buffer_status);

  /* Unlock. */
  pthread_mutex_unlock(&mut);

  /* We're done. */
  close(sd);
  return;

error:
  /* In case of an error, the user is no longer available for us. */
  pthread_mutex_lock(&mut);
  user->status = OFFLINE;
  pthread_mutex_unlock(&mut);

  if (sd > 0)
    close(sd);
  
  return;
}


/*
 * Send a "NEW" request. (=Send a new user to a host)
 */
void send_notify(User * user, char *host)
{
  char buffer_out[200] = "";
  int sd = 0;

  /* Connect. */
  if ((sd = connect_host(host, LINTALK_PORT)) <= 0)
    goto done;

  /* Send a NEW query. (we don't care if it worked or not, so we don't need error handling here) */
  snprintf(buffer_out, 199, "NEW %s\n", user->host);
  send_request(sd, buffer_out);

#ifdef _DEBUG_
  fprintf(stderr, ">> %s", buffer_out);
#endif

done:
  close(sd);
  return;
}


/*
 * Send a list of all users to socket sd
 */
void send_user_list(int sd)
{
  GList *output = NULL;         /* The list of data to send              */
  GList *item = NULL;
  User *user = NULL;
  char buffer[200];
  char buffer_status[2048];

#ifdef _DEBUG_
  fprintf(stderr, "Copying user list. (send_user_list)\n");
#endif

  /* Lock mutex and create a list of all users. */
  pthread_mutex_lock(&mut);
  item = cfg.users;
  while (item != NULL) {
    user = item->data;

    snprintf(buffer, 199, "NAME=%s\n", user->name);
    output = g_list_append(output, strdup(buffer));
    snprintf(buffer, 199, "PHONE=%s\n", user->phone);
    output = g_list_append(output, strdup(buffer));
    snprintf(buffer, 199, "HOST=%s\n", user->host);
    output = g_list_append(output, strdup(buffer));
    user_status_int_to_char(user->status, buffer_status);
    snprintf(buffer, 199, "STATUS=%s\n", buffer_status);
    output = g_list_append(output, strdup(buffer));

    item = item->next;
  }
  pthread_mutex_unlock(&mut);

  /* Now we send the list to the client. */
  item = output;
  while (item != NULL) {
#ifdef _DEBUG_
    fprintf(stderr, ">> %s", (char *) item->data);
#endif

    send(sd, item->data, strlen(item->data), 0);
    item = item->next;
  }

#ifdef _DEBUG_
  fprintf(stderr, "Cleaning up outgoing list. (send_user_list)\n");
#endif

  /* Cleanup list. */
  item = output;
  while (item != NULL) {
    free(item->data);
    item = item->next;
  }
  g_list_free(output);
}
