/* The LinTalk Project
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h>

#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h> 
#include <string.h>

#include <glib.h>

#include "globals.h"
#include "network.h"
#include "server.h"
#include "users.h"
#include "banlist.h"

//FIXME
#include "gui_talk.h"
//FIXME END

extern pthread_mutex_t  mut;
extern Cfg cfg;


/***************************************************************************************
 * BELOW FOLLOW THE STATICS
 ***************************************************************************************/
/*
 * Sets the available status.
 */
static void set_host_status(char *host, short int status)
{
  User *user = NULL;
  
  if (!host)
    return;
  
  /* We'll touch the user list, so lock. */
  pthread_mutex_lock(&mut);
  
  /* We want to find the host in the user list. */
  if ((user = user_find_from_host(cfg.users, host)))
    user->status = status;
  
  pthread_mutex_unlock(&mut);
}


/*
 * Check if the given host is a valid lintalk host and eventually add it to our known-hosts list.
 */
static void add_host(char *host)
{
  User *user = NULL;
  
  /* Create new user. */
  user = user_create("", "", host, NULL);
  
  /* Check if the user is valid. */
  receive_who(user, user->host);
  
  /* And eventually add it to our list. */
  if (user->status != OFFLINE) {
    pthread_mutex_lock(&mut);
    
    /* If that user is me, don't add. */
    if (strcmp(cfg.local.host, host) == 0) {
      user_free(user);
      pthread_mutex_unlock(&mut);
      return;
    }
    
    /* If the user is already in our list, do nothing. */
    if (user_find_from_host(cfg.users, host)) {
      user_free(user);
      pthread_mutex_unlock(&mut);
      return;
    }
    
    user->new = TRUE;
    cfg.users = g_list_append(cfg.users, user);
    
    pthread_mutex_unlock(&mut);
    return;
  }
  
  user_free(user);
}


/*
 * Receive a message from a remote host.
 */
static void message_receive (int psd, char *host)
{
  Historyentry *histentry = NULL;
  User *user = NULL;
  char buffer_in[2048] = "";
  
  if ( host == NULL )
    return;

  /* Find the user from whom we received something in the user list. */
  pthread_mutex_lock (&mut);
  user = user_find_from_host(cfg.users, host);
  pthread_mutex_unlock (&mut);
  
  if (!user)
    return;
  
  /* Receive a message. */
  get_line (psd, buffer_in, 2047);
#ifdef _DEBUG_
  printf ("<< %s\n", buffer_in);
#endif
  
  /* Append it to the user history. */
  histentry = malloc(sizeof(Historyentry));
  histentry->user = 1;
  histentry->username = strdup(user->name);
  histentry->text = strdup(buffer_in);
  
  pthread_mutex_lock(&mut);
  user->history = g_list_append(user->history, histentry);
  pthread_mutex_unlock(&mut);
  
  /* FIXMEFIXME: This is from the GUI and shouldn't be in this file. */
  /* The problem is that the GUI needs a signal when a message has been sent. */
  /* I'm too lazy to work this out now. */
  talk_update_last(user);
}
/***************************************************************************************
 * END OF STATICS
 ***************************************************************************************/


// FIXMEFIXME: Create a function server_init to break this down a bit.
/*
 * Accepts connections and creates new threads for new connections.
 */
void *
serverloop (void *psd)
{
  struct    protoent  *ptrp;             /* pointer to a protocol table entry  */
  struct    sockaddr_in server_address;  /* structure to hold server's address */
  struct    sockaddr_in client_address;  /* structure to hold client's address */
  int       sd, sd2;                     /* socket descriptors                 */
  int       port;                        /* protocol port number               */
  int       alen;                        /* length of address                  */
  struct    timeval time;                /* Wait time between accepts.         */
  fd_set    read_fds;
  pthread_t tid;                         /* variable to hold thread ID         */
  
  pthread_mutex_init (&mut, NULL);       /* Init mutex                         */
  
#ifdef _DEBUG_
  printf ("serverloop(): Requested server address is: %s\n", cfg.local.host);
#endif
  
  /* Clear sockaddr structure. */
  memset (&server_address, 0, sizeof(struct sockaddr_in));
  /* Set protocol family to Internet. */
  server_address.sin_family = AF_INET;
  /* Set the local IP address. */
  inet_pton(AF_INET, cfg.local.host, &server_address.sin_addr.s_addr);
  
  port = LINTALK_PORT;                                        /* use default port number    */
  server_address.sin_port = htons((u_short)port);

  /* Map TCP transport protocol name to protocol number. */
  if ( (int)(ptrp = getprotobyname ("tcp")) == 0 ) {
    fprintf (stderr, "serverloop(): Cannot map \"tcp\" to protocol number!");
    /* Send shutdown signal. */
    pthread_mutex_lock (&mut);
    cfg.shutdown = 1;
    pthread_mutex_unlock (&mut);
    pthread_exit(0);
  }
  
  /* Create a socket. */
  sd = socket (PF_INET, SOCK_STREAM, ptrp->p_proto);
  if (sd < 0) {
    fprintf (stderr, "serverloop(): Socket creation failed.\n");
    /* Send shutdown signal. */
    pthread_mutex_lock (&mut);
    cfg.shutdown = 1;
    pthread_mutex_unlock (&mut);
    pthread_exit(0);
  }
  
#ifdef _DEBUG_
  printf ("serverloop(): Server address is: %s\n", inet_ntoa(server_address.sin_addr));
#endif
  
  /* Bind a local address to the socket. */
  if ( bind (sd, (struct sockaddr *)&server_address, sizeof(server_address)) < 0 ) {
    fprintf (stderr, "serverloop(): Bind to address %s failed.\n", inet_ntoa(server_address.sin_addr));
    /* Send shutdown signal. */
    pthread_mutex_lock (&mut);
    cfg.shutdown = 1;
    pthread_mutex_unlock (&mut);
    pthread_exit(0);
  }
  
  /* Specify a size of request queue. */
  if ( listen (sd, QLEN) < 0 ) {
    fprintf (stderr, "serverloop(): Listen failed!\n");
    /* Send shutdown signal. */
    pthread_mutex_lock (&mut);
    cfg.shutdown = 1;
    pthread_mutex_unlock (&mut);
    pthread_exit(0);
  }

  alen = sizeof(client_address);
  
  pthread_mutex_lock (&mut);
  /* So everything worked well, we tell everybody our socket address. */
  /* The parent process needs that to eventually close the server. */
  cfg.serversd = sd;
  /* Create the banlist. */
  cfg.banlist = banlist_create();
  pthread_mutex_unlock (&mut);
  
  /* Main server loop - accept and handle requests. */
  fprintf (stderr, "Server up and running.\n");
#ifdef _DEBUG_
  printf ("serverloop(): Waiting for contact...\n");
#endif
  
  while (!cfg.shutdown) {
    /* If the user chose not to be online, don't accept connections. */
    if (cfg.local.status == OFFLINE) {
      usleep (500);
      continue;
    }
    
    /* Set the accept-timeout in seconds. */
    time.tv_sec = 1;
    time.tv_usec = 0;
    
    /* We're listening on one socket only. */
    FD_ZERO(&read_fds);
    FD_SET(sd, &read_fds);
    
    /* Are there waiting connections? */
    if (!select(sd + 1, &read_fds, NULL, NULL, &time))
      continue;
    
    /* Accept new connections. */
    if ( (sd2 = accept(sd, (struct sockaddr *)&client_address, &alen)) < 0 ) {
      fprintf (stderr, "serverloop(): Accept failed! (No sockets left?)\n");
      continue;
    }

    /* If the ip is on the ban-list, drop the request. */
    if (!banlist_check(cfg.banlist, &client_address)) {
      close (sd2);
      continue;
    }
    
    /* Create a new thread that will handle the incoming connection. */
#ifdef _DEBUG_
    printf ("serverloop(): Incoming connection...\n");
#endif
    pthread_create (&tid, NULL, serverthread, (void*)sd2);
  }
  
  banlist_destroy(cfg.banlist);
  
  pthread_exit(0);
}


/*
 * Handles an incoming connection.
 * psd = socket descriptor of the connection.
 */
void *
serverthread (void *psd)
{
  int sd;
  char buffer_in[200];       /* Buffer for strings the server receives. */
  char buffer_out[200];      /* Buffer for strings the server sends.    */
  char remotehost[200];
  User *user = NULL;
  
  sd = (int)psd;
  
  /* Read one line. */
  if( get_line (sd, buffer_in, sizeof(buffer_in)) <= 0 ) {
    fprintf (stderr, "serverthread(): Error while receiving request.\n");
    goto done;
  }
#ifdef _DEBUG_
  printf ("<< %s\n", buffer_in);
#endif

  /* Check request. */
  /* FIXMEFIXME: Create a new procedure for all this. */
  /* FIXMEFIXME: Also, there should be nicer sanity/security checks. */
  if (strncmp(buffer_in, "GET ALL", 7) == 0) {
    send_user_list(sd);
    
    /* Prepare answer. */
    snprintf(buffer_out, 199, "> 201 Send complete.\n");
  }
  else if ( memcmp (buffer_in, "WHO\0", 4) == 0
    || strncmp (buffer_in, "WHO ", 4) == 0
    || strncmp (buffer_in, "NEW ", 4) == 0 )
  {
    /* Prepare answer. */
    if ( strncmp (buffer_in, "NEW ", 4) == 0 )
      snprintf (buffer_out, 199, "> 203 NEW command accepted.\n");
    else {
      char status[2048];
      user_status_int_to_char(cfg.local.status, status);
      snprintf (buffer_out, 199, "NAME=%s\nPHONE=%s\nHOST=%s\nSTATUS=%s\n> 202 Send complete.\n", cfg.local.name, cfg.local.phone, cfg.local.host, status);
    }
  }
  else if ( strncmp (buffer_in, "DOWN ", 5) == 0 )
  {
    /* Set the hosts available status to zero. */
    set_host_status(buffer_in + 5, 0);
    
    /* Prepare answer. */
    snprintf (buffer_out, 199, "> 205 DOWN command accepted.\n");
  }
  else if ( strncmp (buffer_in, "MSG ", 4) == 0 ) {
    strncpy (remotehost, buffer_in + 4, 194);
    
    message_receive (sd, remotehost);
    snprintf (buffer_out, 199, "> 301 Message ACK.\n");
  }
  else {
    /* Our default answer. */
    snprintf (buffer_out, 199, "> 500 Invalid request.\n");
  }
  
#ifdef _DEBUG_
  printf (">> %s", buffer_out);
#endif
  /* Send answer to client. */
  send (sd, buffer_out, strlen(buffer_out), 0);
  
  /* Do I have this user in my known hosts? If yes, update his available status, otherwise add him to the list. */
  if (strlen(buffer_in) >= 5) {
    strncpy(remotehost, buffer_in + 4, 194);
    
    pthread_mutex_lock(&mut);
    user = user_find_from_host(cfg.users, remotehost);
    pthread_mutex_unlock(&mut);
    
    if (!user)
      add_host(remotehost);
    else
      set_host_status(remotehost, 1);
  }
  
done:
  close (sd);
  pthread_exit (0);
}
