/* The LinTalk Project
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h> 

#include <pthread.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include "globals.h"

#include "shutdown.h"
#include "network.h"
#include "users.h"


extern pthread_mutex_t  mut;
extern Cfg cfg;


/***************************************************************************************
 * BELOW FOLLOW THE STATICS
 ***************************************************************************************/
/*
 * Send "down" message to all known hosts.
 */
static void
send_down ()
{
  GList *item = NULL;
  User *user = NULL;
  char buffer_out[200] = "";
  char host[2048] = "";
  int sd = 0;
  
  /* Parse through the user list. */
  item = cfg.users;
  while (item != NULL) {
    user = item->data;
    
    if (user->status == OFFLINE) {
      item = item->next;
      continue;
    }
    
    /* If the user is available, we make a backup of his hostname. */
    strncpy(host, user->host, 2047);
    
    /* Is this our own hostname? Then just go on with the next user. */
    if ( strcmp (host, cfg.local.host) == 0 ) {
      item = item->next;
      continue;
    }

    /* Connect. We stay locked, because we don't want other threads to connect anymore now. */
    /* If we can't connect, go on with the next user. */
    if ( (sd = connect_host (host, LINTALK_PORT)) <= 0 ) {
      item = item->next;
      continue;
    }

    /* Prepare DOWN message. */
    snprintf (buffer_out, 199, "DOWN %s\n", cfg.local.host);
    
    /* Send the DOWN message. Still locked. In case of an error */
    /* we will close the socket and go on with the next user. */
    if ( !send_request (sd, buffer_out) ) {
      close (sd);
      item = item->next;
      continue;
    }
    
    /* Close socket. */
    close (sd);
    
    /* Next user. */
    item = item->next;
  }
  
  /* We have cleaned up, but we'll NOT unlock. */
  return;
}
/***************************************************************************************
 * END OF STATICS
 ***************************************************************************************/


/*
 * Will be called all 500 milliseconds to check if the server has been shut down.
 */
int
check_if_server_shut (void *null)
{
  if ( cfg.shutdown == 1 ) {
    /* GTK must be thread save. */
    gdk_threads_enter ();
    printf("Shutting down... ");
    fflush(stdin);
    
    /* We modify the user variables, so we lock. */
    pthread_mutex_lock(&mut);
  
    /* Notify neighbors of down request. */
    send_down();

    /* All variables have been cleaned up. */
    pthread_mutex_unlock(&mut);
    
    /* Close the server's socket. */
    close(cfg.serversd);
    
    /* End application. */
    gdk_threads_leave();
    gtk_main_quit();
    
    printf("Goodbye.\n");
    
    return (0);
  }
  
  /* GTK must be thread save. */
  gdk_threads_enter ();
  /* This is the next shutdown-check interval. */
  gtk_timeout_add (500, check_if_server_shut, NULL);
  gdk_threads_leave ();
  
  return (0);
}
