/* The LinTalk Project
 * (c)2002 by Samuel Abels (sam@manicsadness.com)
 * This project's homepage is: http://software.manicsadness.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h> 
#include <string.h> 

#include "support.h"
#include "callbacks.h"


#include "users.h"
#include "network.h"
#include "lib_configfile.h"


/***************************************************************************************
 * STATICS
 ***************************************************************************************/
/***************************************************************************************
 * END STATICS
 ***************************************************************************************/


/*
 * Create a new user
 */
User *user_create(const gchar *name, const gchar *phone, const gchar *host, const gchar *groupname)
{
  /* Init user variable. */
  User *user = malloc(sizeof(User));
  
  user->name = strdup(name);
  user->phone = strdup(phone);
  user->host = strdup(host);
  user->group = 0;
  if (groupname) {
    Group *group = group_find_from_name(groupname);
    if (group) {
      user->group = group;
    }
    else {
      user->group = group_add(strdup(groupname));
    }
  }
  
  user->updategui = FALSE;
  
  user->new = TRUE;
  user->forward = FALSE;
  user->status = OFFLINE;
  user->talk = NULL;
  user->history = NULL;
  user->outqueue = NULL;
  
  return (user);
}


/*
 * Free one user.
 */
void user_free(User * user)
{
  if(!user)
    return;
  
  if (user->name != NULL)
    free(user->name);
  
  if (user->phone != NULL)
    free(user->phone);
  
  if (user->host != NULL)
    free(user->host);
  
  free(user);
}


/*
 * Free all users.
 */
void users_free_all(GList * userlist)
{
  User * user = NULL;
  
  while (userlist) {
    user = (User *)userlist->data;
    if (user)
      user_free(user);
    userlist = userlist->next;
  }
  
  g_list_free(userlist);
  
  return;
}

/*
 * Returns NULL if the host was not found in the userlist, otherwise the user.
 */
User * user_find_from_host(GList * userlist, char *host)
{
  GList *item = NULL;
  User *user = NULL;

  if (!host)
    return (NULL);

  /* Parse through the user list. */
  item = userlist;
  while (item != NULL) {
    user = item->data;

    /* Match? */
    if (strcmp(user->host, host) == 0)
      return(user);

    item = item->next;
  }

  return (NULL);
}


/*
 * Return the corresponding integer value of the status.
 */
int user_status_char_to_int(const char *status)
{
  typedef struct {
    char *name;
    int number;
  } Status;
  const Status list[] = {
    {"OFFLINE", OFFLINE},
    {"AVAILABLE", AVAILABLE},
    {NULL, 0}
  };
  int i = 0;
  
  do {
    if (strcmp(list[i].name, status) == 0)
      return(list[i].number);
  } while (list[++i].name);
  
  return(INVALID);
}


/*
 * Copy the corresponding integer value of the status.
 */
void user_status_int_to_char(int status, char *statusstring)
{
  typedef struct {
    char *name;
    int number;
  } Status;
  const Status list[] = {
    {"OFFLINE", OFFLINE},
    {"AVAILABLE", AVAILABLE},
    {NULL, 0}
  };
  int i = 0;
  
  do {
    if (list[i].number == status) {
      strcpy(statusstring, list[i].name);
      return;
    }
  } while (list[++i].name);
}
